package org.openxma.dsl.reference.ui.layouting.server;

import at.spardat.xma.page.*;
import at.spardat.xma.component.*;
import at.spardat.xma.mdl.*;
import at.spardat.enterprise.util.*;
import at.spardat.xma.mdl.simple.*;

/**
 * This class implements the generic server part of FieldParts.
 * It implements the models of the page.
 *
 */
public abstract class FieldPartsGen extends PageServer {
    ISimpleWM customer1_lastName;
    ISimpleWM customer2_lastName;
    WModel[] widgetModels;


    /**
     * Constructs the FieldPartsGen.
     * This constructor is called by the runtime. Do not remove!
     *
     * @param component the component this page belongs to
     */
    public FieldPartsGen(ComponentServer component) {
        super(component, false);
        createModels();
    }

    /**
     * Create and initialize the WidgetModels of the page.
     */
    public void createModels() {
        customer1_lastName = new SimpleWM((short) 0, Types.T_STRING, this);
        customer2_lastName = new SimpleWM((short) 1, Types.T_STRING, this);

        widgetModels = new WModel[] { (WModel) customer1_lastName, (WModel) customer2_lastName };
    }

    /**
     * Returns an array of all WidgetModels of this page.
     * @return array of WidgetModels. The returned array is not a copy, therefore
     *          it must not be modified.
     */
    public WModel[] getWModels() {
        return widgetModels;
    }

    /**
     * Replaces the model array generated by guidesigner.
     * May only be called by the runtime. Do not call this method!
     */
    protected void setWModels(WModel[] models) {
        widgetModels=models;
    }

    /**
     * Returns a unique id to identify the type of this Page.
     * @return the typeId
     */
    public short getTypeId() {
        return Layouting.FieldPartsClass;
    }

    /**
     * Returns the component this page belongs to, casted to its actual type.
     * @return The Component. Never null.
     */
    public Layouting getTypedComponent() {
        return (Layouting) getComponent();
    }

    /**
     * Copies values of all attached widget models to attributes of the BusinessData instance variables.
     * Copies them for this page and all contained NotebookPages and statically embedded Pages
     * (embedded via XMAContainer in guidesigner)
     */
    public void modelToInstanceData() {
        atomicWMToBusinessData(getTypedComponent().getCustomer1(), getTypedComponent().getCustomer2());
    }

    /**
     * Copies attribute values of the BusinessData instance variables into their attached widget models.
     * This includes simple widget models and tables.
     * Copies them for this page and all contained NotebookPages and statically embedded Pages
     * (embedded via XMAContainer in guidesigner)
     */
    public void instanceDataToModel() {
        businessDataToAtomicWM(getTypedComponent().getCustomer1(), getTypedComponent().getCustomer2());
    }

    /**
     * Copies values of widget models to attributes of the provided BusinessData objects.
     */
    public void atomicWMToBusinessData(org.openxma.dsl.reference.dto.CustomerView customer1, org.openxma.dsl.reference.dto.CustomerView customer2) {
        customer1.setLastName(customer1_lastName.toString());
        customer2.setLastName(customer2_lastName.toString());
    }

    /**
     * Copies attribute values of the provided BusinessDatas into widget models.
     */
    public void businessDataToAtomicWM(org.openxma.dsl.reference.dto.CustomerView customer1, org.openxma.dsl.reference.dto.CustomerView customer2) {
        customer1_lastName.set(customer1.getLastName());
        customer2_lastName.set(customer2.getLastName());
    }

    /**
     * Sets the given subpage. The method automatically chooses the
     * correct member variable to set.
     * @param child the subpage to set.
     */
    public void setChild(PageServer child) {
       // no subpages - nothing to do
    }

    /**
     * Removes the given subpage. The method automatically chooses the
     * correct member variable to clear.
     * @param child the subpage to set.
     */
    public void removeChild(PageServer child) {
       // no subpages - nothing to do
    }

}
