package org.openxma.dsl.reference.xma.customeredit.client;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import at.spardat.xma.component.EmbeddableComponent;
import at.spardat.xma.component.ComponentProperty;
import at.spardat.xma.session.XMASessionClient;

/**
 *  Customer Edit Comop
 *  
 * This class implements the generic client part of CustomerEdit.
 * It provides the mapping between typeIds and classnames
 * of all pages in the component.
 *
 */
abstract public class CustomerEditGen extends EmbeddableComponent implements ICustomerEdit {

    /**
     * the mode of the component
     */
    protected String mode = "default";

    /**
     * the id of the selected customer
     */
    protected String selectedCustomerId;

    /**
     * firstname of the selected customer
     */
    protected String selectedCustomerFirstName;

    /**
     * lastname of the selected customer
     */
    protected String selectedCustomerLastName;

    /**
     * uncommentedProperty - property
     */
    protected String uncommentedProperty;

    /** describes all component properties */
    private ComponentProperty[] propDes;

    /**
     * Initializes the CustomerEditGen. This method is called
     * by the XMA-Runtime.
     *
     * @param session the XMASession, this CustomerEditGen belongs to.
     */
    public CustomerEditGen (XMASessionClient session) {
        super (session, false);
    }

    /** the unique typeId for the page CustomerSearchForm */
    public static final short CustomerSearchFormClass = 0;
    /** the unique typeId for the page CustomerEditForm */
    public static final short CustomerEditFormClass = 1;
    /** the unique typeId for the page Page1 */
    public static final short Page1Class = 2;
    /** the unique typeId for the page Page2 */
    public static final short Page2Class = 3;

    /**
    * Array of the classnames of all pages in
    * this component. The typeIds are used as indices
    * into this array.
     */
    static final String[] modelClasses = {
        "CustomerSearchForm",
        "CustomerEditForm",
        "Page1",
        "Page2"
    };

    /**
    * Gets the short classname for the given typeId of
    * a page in this component. The short classname does
    * not contain the package.
    *
    * @param typeId the unique typeId
    * @return the short classname
     */
    static public String getShortModelClass(short typeId) {
        return modelClasses[typeId];
    }

    /**
    * Gets the full classname for the given typeId of
    * a page in this component.
    *
    * @param typeId the unique typeId
    * @return the full classname
     */
    public String getModelClass(short typeId) {
        return "org.openxma.dsl.reference.xma.customeredit.client."+modelClasses[typeId];
    }

    /**
     * Invokes the CustomerEdit. This method is a default
     * implementation, which invokes the first dialog page of the component.
     * Modeled init functions are called by this method.
     *
     * @param parentComposite a composite of the calling page. In most
     *     cases it will be a shell. It my be null.
     */
    public void invoke(Composite parentComposite) {
        CustomerSearchForm customerSearchForm;
        if(parentComposite instanceof Shell) {
            customerSearchForm = new CustomerSearchForm(this,(Shell)parentComposite);
        } else {
            customerSearchForm = new CustomerSearchForm(this);
        }
        customerSearchForm.invoke();
    }

    /**
     * Gives the URI of the help for this component
     * This URI is relative to the download-URI of the component
     * @return the relative URI of the help for this component
     */
    public String getHelpUri() {
        return "help/org/openxma/dsl/example/simple/Example.html";
    }

    /**
     * the mode of the component
     */
    protected String getMode() {
        return mode;
    }

    /**
     * the mode of the component
     */
    public void setMode(String mode) {
        String oldmode = this.mode;
        this.mode = mode;
        notifyPropertyChange("mode",oldmode,mode);
    }

    /**
     * the id of the selected customer
     */
    public String getSelectedCustomerId() {
        return selectedCustomerId;
    }

    /**
     * the id of the selected customer
     */
    protected void setSelectedCustomerId(String selectedCustomerId) {
        String oldselectedCustomerId = this.selectedCustomerId;
        this.selectedCustomerId = selectedCustomerId;
        notifyPropertyChange("selectedCustomerId",oldselectedCustomerId,selectedCustomerId);
    }

    /**
     * firstname of the selected customer
     */
    public String getSelectedCustomerFirstName() {
        return selectedCustomerFirstName;
    }

    /**
     * firstname of the selected customer
     */
    protected void setSelectedCustomerFirstName(String selectedCustomerFirstName) {
        String oldselectedCustomerFirstName = this.selectedCustomerFirstName;
        this.selectedCustomerFirstName = selectedCustomerFirstName;
        notifyPropertyChange("selectedCustomerFirstName",oldselectedCustomerFirstName,selectedCustomerFirstName);
    }

    /**
     * lastname of the selected customer
     */
    public String getSelectedCustomerLastName() {
        return selectedCustomerLastName;
    }

    /**
     * lastname of the selected customer
     */
    protected void setSelectedCustomerLastName(String selectedCustomerLastName) {
        String oldselectedCustomerLastName = this.selectedCustomerLastName;
        this.selectedCustomerLastName = selectedCustomerLastName;
        notifyPropertyChange("selectedCustomerLastName",oldselectedCustomerLastName,selectedCustomerLastName);
    }

    /**
     * uncommentedProperty - property
     */
    protected String getUncommentedProperty() {
        return uncommentedProperty;
    }

    /**
     * uncommentedProperty - property
     */
    protected void setUncommentedProperty(String uncommentedProperty) {
        String olduncommentedProperty = this.uncommentedProperty;
        this.uncommentedProperty = uncommentedProperty;
        notifyPropertyChange("uncommentedProperty",olduncommentedProperty,uncommentedProperty);
    }

    /**
     * @return descriptors of the properties of this component
     */
    protected ComponentProperty[] getPropDes() {
        if(propDes==null) {
            propDes = new ComponentProperty[5];
            propDes[0]=new ComponentProperty("mode",String.class,this,CustomerEditGen.class);
            propDes[1]=new ComponentProperty("selectedCustomerId",String.class,this,CustomerEditGen.class);
            propDes[2]=new ComponentProperty("selectedCustomerFirstName",String.class,this,CustomerEditGen.class);
            propDes[3]=new ComponentProperty("selectedCustomerLastName",String.class,this,CustomerEditGen.class);
            propDes[4]=new ComponentProperty("uncommentedProperty",String.class,this,CustomerEditGen.class);
        }
        return propDes;
    }

}
